# coding=utf-8
import os
from unittest import TestCase
from tempfile import mktemp

from mock import Mock, patch

# import dummydata.model3

from earthdiagnostics.diagnostic import DiagnosticVariableOption
from earthdiagnostics.frequency import Frequencies
from earthdiagnostics.general.timemean import MonthlyMean
from earthdiagnostics.modelingrealm import ModelingRealms


class TestMonthlyMean(TestCase):
    def setUp(self):
        self.var_file = mktemp(".nc")
        self.data_manager = Mock()

        self.diags = Mock()
        self.diags.model_version = "model_version"
        self.diags.config.experiment.get_chunk_list.return_value = (
            ("20010101", 0, 0),
            ("20010101", 0, 1),
        )

        self.monmean = MonthlyMean(
            self.data_manager,
            "20000101",
            1,
            1,
            ModelingRealms.atmos,
            "ta",
            "freq",
            "",
        )

    def tearDown(self):
        if os.path.exists(self.var_file):
            os.remove(self.var_file)

    def fake_parse(self, value):
        return value

    @patch.object(DiagnosticVariableOption, "parse", fake_parse)
    def test_generate_jobs(self):
        jobs = MonthlyMean.generate_jobs(
            self.diags, ["diagnostic", "atmos", "var", "monthly"]
        )
        self.assertEqual(len(jobs), 2)
        self.assertEqual(
            jobs[0],
            MonthlyMean(
                self.data_manager,
                "20010101",
                0,
                0,
                ModelingRealms.atmos,
                "var",
                Frequencies.monthly,
                "",
            ),
        )
        self.assertEqual(
            jobs[1],
            MonthlyMean(
                self.data_manager,
                "20010101",
                0,
                1,
                ModelingRealms.atmos,
                "var",
                Frequencies.monthly,
                "",
            ),
        )

        jobs = MonthlyMean.generate_jobs(
            self.diags, ["diagnostic", "seaice", "var", "mon", "grid"]
        )
        self.assertEqual(len(jobs), 2)
        self.assertEqual(
            jobs[0],
            MonthlyMean(
                self.data_manager,
                "20010101",
                0,
                0,
                ModelingRealms.seaIce,
                "var",
                Frequencies.monthly,
                "grid",
            ),
        )
        self.assertEqual(
            jobs[1],
            MonthlyMean(
                self.data_manager,
                "20010101",
                0,
                1,
                ModelingRealms.seaIce,
                "var",
                Frequencies.monthly,
                "grid",
            ),
        )

        with self.assertRaises(Exception):
            MonthlyMean.generate_jobs(self.diags, ["diagnostic"])

        with self.assertRaises(Exception):
            MonthlyMean.generate_jobs(
                self.diags, ["diagnostic", "0", "0", "0", "0", "0", "0", "0"]
            )

    def test_str(self):
        self.assertEqual(
            str(self.monmean),
            "Calculate monthly mean Startdate: 20000101 Member: 1 Chunk: 1 "
            "Variable: atmos:ta Original frequency: freq Grid: ",
        )
